/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Show Related Links extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2004-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

const DEBUG = true;
const AWS_ALEXA_URL = "http://www.amazon.com/gp/browse.html/ref=sc_fe_c_1_3435361_7/102-0625751-1833753?%5Fencoding=UTF8&node=12782661&no=3435361&me=A36L942TSJ2AJA";


var gTabbedBrowser;
var gSidebar;
var gStrBundle;
var gClickedURL = null;
var gIsBusy = false;


function init() 
{
  gTabbedBrowser = parent.document.getElementById("content");
  gSidebar = document.getElementById("related-info-display");
  gStrBundle = document.getElementById("string-bundle");
  gIsBusy = true;

  updateCurrentURLDisplay();
  populateSidebar();
}


function updateCurrentURLDisplay() 
{
  var currentURL = getCurrentURL();
  var urlDisplay = document.getElementById("url-display");
  urlDisplay.value = String.format("%s %s", gStrBundle.getString("for"), currentURL);
  urlDisplay.tooltipText = currentURL;
}


function populateSidebar() 
{
  try {
    callAlexaService();
  }
  catch (e) {
    log(String.format("Exception thrown from function callAlexaService():\n%s", e));
  }
}


function callAlexaService()
{
  // The following URI from the WSDL file doesn't work (causes an
  // "AWS.MinimumParameterRequirement" error when using it as the value 
  // of the web service endpoint URI:
  //
  //    http://awis.amazonaws.com/onca/soap?Service=AlexaWebInfoService
  //
  // The URI used, and the one that works, is from the AWIS beta version of 
  // the WSDL file.
  const ENDPOINT = "http://soap.amazon.com/onca/soap?Service=AlexaWebInfoService";
  const TARGET_NS = "http://webservices.amazon.com/AWSAlexa/2005-07-11";
  const METHOD_NAME = "UrlInfo";

  var alexaService = new SOAPCall();
  alexaService.transportURI = ENDPOINT;

  var key = getAWSAccessKeyID();
  var service = "AlexaWebInfoService";
  var operation = METHOD_NAME;
  var timestamp = dtGetXSDDateTimeString(new Date());
  var signature = b64_hmac_sha1(key, service + operation + timestamp);

  log(String.format("Relatedlinks: timestamp=%S", timestamp));

  var params = [];
  params[0] = new SOAPParameter(key, "AWSAccessKeyId");
  params[1] = new SOAPParameter(encodeURIComponent(signature), "Signature");
  params[2] = new SOAPParameter(timestamp, "Timestamp");
  params[3] = new SOAPParameter(getCurrentURL(), "Url");
  params[4] = new SOAPParameter("RelatedLinks,Rank,ContactInfo", "ResponseGroup");

  alexaService.encode(0, METHOD_NAME, TARGET_NS, 0, null, params.length, params);
  toggleProgressIndicator();
  alexaService.asyncInvoke(function (response, alexaService, error) 
                           {
			     toggleProgressIndicator();
                             try {
                               handleResponse(response, error);
                             } catch (e) {
                               displayError(e);
			       gIsBusy = false;
                               return false;
                             }
                             processAlexaResults(response.body);
                           });
}


function handleResponse(response, error) 
{
  if (error != 0) {
    throw gStrBundle.getString("errorServiceFailure");
  }
  else {
    if (! response) {
      throw gStrBundle.getString("errorServiceUnavailable");
    }
    if (response.fault) {
      throw (String.format("%s: %s [%s: %s]", 
			   gStrBundle.getString("serviceFault"), 
			   response.fault.faultString, 
			   gStrBundle.getString("code"), 
			   response.fault.faultCode));
    }
  }
}


function processAlexaResults(resultDOM) 
{
  if (! resultDOM) {
    log("Error in processAlexaResults(): resultDOM cannot be null!");
    return false;
  }

  if (DEBUG) {
    gSidebar.appendChild(resultDOM);
  }

  // AWS error - e.g. insufficient params, bad signature
  if (resultDOM.getElementsByTagName("Errors").length > 0) {
    var err = resultDOM.getElementsByTagName("Error").item(0).firstChild.firstChild.data;
    displayError(String.format("%s: %s", gStrBundle.getString("error"), err));
    gIsBusy = false;
    return false;
  }

  // Invalid AWS subscription ID or malformed SOAP request
  // If the UrlInfo request was valid, then node value of <IsValid> is "True"
  if (resultDOM.getElementsByTagName("IsValid").item(0).firstChild.data == "False") {
    displayError(gStrBundle.getString("errorInvalidRequest"));
    gIsBusy = false;
    return false;
  }

  var relatedLinks = resultDOM.getElementsByTagName("RelatedLink");
  if (relatedLinks.length == 0) {
    displayError(gStrBundle.getString("errorNoResults"));
    gIsBusy = false;
    return false;
  }

  listRelatedLinks(resultDOM);
  listRelatedInfo(resultDOM);
  listContactInfo(resultDOM);
  displayResultProvider();

  if (gIsBusy) {
    gIsBusy = false;
  }
}


function listRelatedLinks(resultDOM) 
{
  var relatedLinksNodeList = resultDOM.getElementsByTagName("RelatedLink");
  var titleNodeList = resultDOM.getElementsByTagName("Title");
  var navigableUrlNodeList = resultDOM.getElementsByTagName("NavigableUrl");
  if (! relatedLinksNodeList) {
    return;
  }

  var relatedLinksHdg = document.createElement("description");
  relatedLinksHdg.className = "hdg";
  relatedLinksHdg.setAttribute("value", gStrBundle.getString("subhdgRelatedLinks"));
  gSidebar.appendChild(relatedLinksHdg);

  for (var i = 0; i < relatedLinksNodeList.length; i++) {
    var url = navigableUrlNodeList[i].firstChild.data;
    var anchorNode = document.createElement("description");
    anchorNode.className = "hyperlink";
    anchorNode.tooltipText = url;
    anchorNode.setAttribute("url", url);
    anchorNode.setAttribute("crop", "end");
    anchorNode.setAttribute("value", titleNodeList[i].firstChild.data);
    anchorNode.setAttribute("onclick", "relatedLink_click(event)");
    anchorNode.setAttribute("context", "hyperlink-context-menu");
    gSidebar.appendChild(anchorNode);
  }

  gSidebar.appendChild(document.createElement("separator"));
}


function relatedLink_click(event)
{
  var url = event.target.getAttribute("url");
  if (url) {
    gClickedURL = url;
  }

  // event.button values:
  //  0 - left mouse button
  //  2 - right mouse button
  if (event.button == 0) {
    gotoURL(url);
  }
}


function listRelatedInfo(resultDOM) 
{
  var rankNode = resultDOM.getElementsByTagName("Rank").item(0);
  if (! rankNode) {
    return;
  }

  var relatedInfoHdg = document.createElement("description");
  relatedInfoHdg.className = "hdg";
  relatedInfoHdg.setAttribute("value", gStrBundle.getString("subhdgRelatedSiteInfo"));
  gSidebar.appendChild(relatedInfoHdg);

  var trafficRanking = document.createElement("description");
  trafficRanking.setAttribute("value", String.format("%s: %s", gStrBundle.getString("trafficRanking"), rankNode.firstChild.data));
  gSidebar.appendChild(trafficRanking);
}


function listContactInfo(resultDOM) 
{
  var contactInfoNode = resultDOM.getElementsByTagName("ContactInfo").item(0);
  if (! contactInfoNode) {
    return;
  }

  // <OwnerName> multiplicity: [0..1]
  var ownerName = getNodeValue(contactInfoNode, "OwnerName");
  if (ownerName) {
    var ownerNameNode = document.createElement("description");
    ownerNameNode.className = "contact-info";
    ownerNameNode.appendChild(document.createTextNode(ownerName));
    gSidebar.appendChild(ownerNameNode);
  }

  // <Street> multiplicity: [1..*]
  var streetAddress = contactInfoNode.getElementsByTagName("Street");
  for (var i = 0; i < streetAddress.length; i++) {
    var streetNode = document.createElement("description");
    streetNode.className = "contact-info";
    streetNode.appendChild(document.createTextNode(streetAddress[i].firstChild.data));
    gSidebar.appendChild(streetNode);
  }

  // <City>, <State>, <PostalCode>, <Country>: each [0..1]
  var city = getNodeValue(contactInfoNode, "City");
  var state = getNodeValue(contactInfoNode, "State");
  var postalCode = getNodeValue(contactInfoNode, "PostalCode");
  var country = getNodeValue(contactInfoNode, "Country");

  if (city || state || postalCode) {
    var physLocNode = document.createElement("description");
    physLocNode.className = "contact-info";
    physLocNode.appendChild(document.createTextNode(city + ", " + state + "  " + postalCode));
    gSidebar.appendChild(physLocNode);
  }

  if (country) {
    var countryNode = document.createElement("description");
    countryNode.className = "contact-info";
    countryNode.appendChild(document.createTextNode(country));
    gSidebar.appendChild(countryNode);
  }

  // Only 1st <PhoneNumber> node retrieved; node multiplicity: [0..*]
  var phoneNumber = getNodeValue(contactInfoNode, "PhoneNumber");
  if (phoneNumber) {
    var phoneNumberNode = document.createElement("description");
    phoneNumberNode.className = "contact-info";
    phoneNumberNode.appendChild(document.createTextNode(phoneNumber));
    gSidebar.appendChild(phoneNumberNode);
  }
}


function getNodeValue(DOMNode, tagName) 
{
  var nodeList = DOMNode.getElementsByTagName(tagName);
  if (!nodeList || nodeList.length == 0) {
    return "";
  }

  var node = nodeList[0];
  if (!node.childNodes || node.childNodes.length == 0) {
    return "";
  }
  return (node.firstChild.data || "");
}


function displayResultProvider() 
{
  var descNode = document.createElement("description");
  var textNode = document.createTextNode(gStrBundle.getString("ack") + " ");
  var linkNode = document.createElementNS("http://www.w3.org/1999/xhtml", "html:a");
  linkNode.className = "hyperlink";
  linkNode.setAttribute("href", "javascript:gotoURL('" + AWS_ALEXA_URL + "')");
  linkNode.appendChild(document.createTextNode(gStrBundle.getString("src")));
  descNode.className = "result-provider";
  descNode.appendChild(textNode);
  descNode.appendChild(linkNode);  // Append <html:a> element
  
  gSidebar.appendChild(document.createElement("separator"));
  gSidebar.appendChild(descNode);
}


function getAWSAccessKeyID()
{
  var cls = Components.classes["@mozilla.org/preferences-service;1"];
  var prefSvc = cls.getService(Components.interfaces.nsIPrefBranch);
  var rv = "07AFVCTXQJ3DKQTAZK02";  // Default used unless user specifies own
  try {
    rv = prefSvc.getCharPref("relatedlinks.aws_access_key");
  } catch (e) { }

  return rv;
}


function reloadSidebar() 
{
  if (gIsBusy) {
    return;  // Don't interrupt sidebar reload in progress.
  }
  gIsBusy = true;

  while (true) {
    var child = gSidebar.firstChild;
    try {
      gSidebar.removeChild(child);
    }
    catch (e if e.code == e.NOT_FOUND_ERR) {
      break;
    } 
    catch (e) {
      log(e);
      break;
    }
  }

  updateCurrentURLDisplay();
    
  populateSidebar();
}


function toggleProgressIndicator()
{
  var progressMsg = document.getElementById("progress-msg");
  var progressBar = document.getElementById("progress-bar");
  if (progressMsg && progressBar) {
    gSidebar.removeChild(progressMsg);
    gSidebar.removeChild(progressBar);
  }
  else {
    progressMsg = document.createElement("description");
    progressBar = document.createElement("progressmeter");
    progressMsg.id = "progress-msg";
    progressBar.id = "progress-bar";
    progressMsg.setAttribute("value", gStrBundle.getString("msgPleaseWait"));
    progressMsg.style.fontWeight = "bold";
    progressBar.setAttribute("mode", "undetermined");
    gSidebar.appendChild(progressMsg);
    gSidebar.appendChild(progressBar);
  }
}


function showHelp()
{
  window.openDialog("chrome://relatedlinks/content/help.xul", "hlp", "modal");
}


function displayError(errorMsg) 
{
  var descNode = document.createElement("description");
  descNode.className = "error";
  descNode.style.color = "red";
  var textNode = document.createTextNode(errorMsg);
  descNode.appendChild(textNode);
  gSidebar.appendChild(descNode);
}


function getCurrentURL() 
{
  return gTabbedBrowser.selectedBrowser.currentURI.spec;
}


function gotoURL(url) 
{
  gTabbedBrowser.selectedBrowser.loadURI(url);
}


function openInNewWindow_command(event)
{
  window.open(gClickedURL);
}


function openInNewTab_command(event)
{
  gTabbedBrowser.addTab(gClickedURL);
}


function hideMenu_keypress(event)
{
  document.getElememtById("hyperlink-context-menu").hidePopup();
}



function log(aMessage) 
{
  if (DEBUG) {
    var consoleSvc = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
    consoleSvc.logStringMessage(aMessage);
  }
}
